const IDTO = global.IDTO || require('./IDTO')
const Joi = global.Joi || require('../../static_dependencies/joi/joi-browser.min.js')
const UID = global.UID || require('../utils/UID')

/**
 * AssetDTO class
 *
 * @author jsb4ch <jsb4ch@jsmex.io>
 * @since 1.0.0
*/
class AssetDTO extends IDTO {
  constructor (uid) {
    super(uid)

    this.alias = ''
    this.asset = ''
    this.isoAlias = ''
  }

  /**
   * AssetDTO iso getter
   *
   * @author jsb4ch <jsb4ch@jsmex.io>
   * @since 1.0.0
  */
  get iso () {
    return this.isoAlias && this.isoAlias.length ? this.isoAlias : this.alias
  }

  /**
   * AssetDTO validation method
   *
   * @author jsb4ch <jsb4ch@jsmex.io>
   * @since 1.0.0
  */
  validate () {
    let data = {
      alias: this.alias,
      asset: this.asset,
      isoAlias: this.isoAlias
    }
    let joiValidDepth = Joi.object()
      .keys({
        alias: Joi.string().required(),
        asset: Joi.string().required(),
        isoAlias: Joi.alternatives().try(
          Joi.string(),
          Joi.string().valid('')
        )
      })

    let result = Joi.validate(data, joiValidDepth)
    if (result.error) {
      throw result.error
    }
    return result.value
  }

  /**
   * AssetDTO toObject method
   *
   * @author jsb4ch <jsb4ch@jsmex.io>
   * @since 1.0.8
   * @returns {Object}
  */
  toObject () {
    return {
      uid: this.uid,
      asset: this.asset,
      alias: this.alias,
      isoAlias: this.isoAlias
    }
  }

  /**
   * AssetDTO fromObject method
   *
   * @author jsb4ch <jsb4ch@jsmex.io>
   * @since 1.0.8
   * @param objectData object
   * @returns AssetDTO
  */
  static fromObject (objectData) {
    let assetDTO = new AssetDTO(UID.create())
    if (objectData.asset) {
      assetDTO.asset = objectData.asset
    }
    if (objectData.alias) {
      assetDTO.alias = objectData.alias
    }
    if (objectData.isoAlias) {
      assetDTO.isoAlias = objectData.isoAlias
    }
    assetDTO.validate()
    return assetDTO
  }
}

module.exports = AssetDTO
