const IDTO = global.IDTO || require('./IDTO')
const Joi = global.Joi || require('../../static_dependencies/joi/joi-browser.min.js')
const UID = global.UID || require('../utils/UID')

/**
 * BalanceDTO class
 *
 * @author jsb4ch <jsb4ch@jsmex.io>
 * @since 1.0.0
*/
class BalanceDTO extends IDTO {
  constructor (uid) {
    super(uid)

    this.exchangeName = ''
    this.createdAt = new Date()
    this.values = {}
  }

  /**
   * BalanceDTO validation method
   *
   * @author jsb4ch <jsb4ch@jsmex.io>
   * @since 1.0.0
  */
  validate () {
    let data = {
      exchangeName: this.exchangeName,
      createdAt: this.createdAt,
      values: this.values
    }
    let joiValidDepth = Joi.object()
      .keys({
        exchangeName: Joi.string(),
        createdAt: Joi.any().required(),
        values: Joi.object().pattern(
          Joi.string(),
          Joi.object().pattern(
            Joi.string().valid('available', 'freeze'),
            Joi.string()
          )
        ).required()
      })

    let result = Joi.validate(data, joiValidDepth)
    if (result.error) {
      throw result.error
    }
    return result.value
  }

  /**
   * BalanceDTO toObject method
   *
   * @author jsb4ch <jsb4ch@jsmex.io>
   * @since 1.0.8
   * @returns {Object}
  */
  toObject () {
    return {
      uid: this.uid,
      exchangeName: this.exchangeName,
      createdAt: this.createdAt,
      values: this.values
    }
  }

  /**
   * BalanceDTO fromObject method
   *
   * @author jsb4ch <jsb4ch@jsmex.io>
   * @since 1.0.8
   * @param objectData object
   * @returns BalanceDTO
  */
  static fromObject (objectData) {
    let balanceDTO = new BalanceDTO(UID.create())
    if (objectData.exchangeName) {
      balanceDTO.exchangeName = objectData.exchangeName
    }
    if (objectData.createdAt) {
      balanceDTO.createdAt = new Date(objectData.createdAt)
    }
    if (objectData.values) {
      balanceDTO.values = objectData.values
    }
    balanceDTO.validate()
    return balanceDTO
  }
}

module.exports = BalanceDTO
