const IDTOArray = global.IDTOArray || require('./IDTOArray')
const KlineDTO = global.KlineDTO || require('../KlineDTO')
const _ = global.lodash || require('../../../static_dependencies/lodash/lodash.js')

/**
  * KlineDTOArray class
  *
  * @author jsb4ch <jsb4ch@jsmex.io>
  * @since 1.0.9
  */
class KlineDTOArray extends IDTOArray {
  constructor (...args) {
    super(...args)
    this.validate()
  }

  /**
  * KlineDTOArray lastKlineTime getter
  *
  * @author jsb4ch <jsb4ch@jsmex.io>
  * @since 1.0.9
  * @returns Date
  */
  get lastKlineTime () {
    if (!this.length) {
      return null
    } else {
      return _.maxBy(
        this,
        function (o) {
          return o.time
        }
      )['time']
    }
  }

  /**
    * DepthDTOArray validate method
    *
    * @author jsb4ch <jsb4ch@jsmex.io>
    * @since 1.0.9
    */
  validate () {
    for (let kline of this) {
      if (!KlineDTO.isValid(kline)) {
        throw new Error('kline is not kline dto')
      }
    }
    return true
  }

  /**
  * KlineDTOArray save method
  *
  * @author jsb4ch <jsb4ch@jsmex.io>
  * @since 1.0.9
  */
  async save (db, filter = x => x) {
    if (!db) {
      throw new Error('invalid db')
    }
    if (typeof filter !== 'function') {
      throw new Error('invalid filter')
    }
    return db.getCollectionRepository('kline').insertMany(this)
  }

  /**
  * KlineDTOArray load method
  *
  * @author jsb4ch <jsb4ch@jsmex.io>
  * @since 1.0.9
  */
  async load (db, query = {}) {
    if (!db) {
      throw new Error('invalid db')
    }
    let res = await db.getCollectionRepository('kline').get(query) || []
    _.merge(this, res)
    return this
  }

  /**
  * KlineDTOArray remove method
  *
  * @author jsb4ch <jsb4ch@jsmex.io>
  * @since 1.0.9
  */
  async remove (db, filter = x => x) {
    if (!db) {
      throw new Error('invalid db')
    }
    if (typeof filter !== 'function') {
      throw new Error('invalid filter')
    }
    await db.getCollectionRepository('kline').remove(this)
    return this.splice()
  }

  /**
    * KlineDTOArray toObject method
    *
    * @author jsb4ch <jsb4ch@jsmex.io>
    * @since 1.0.13
    */
  toObject () {
    let objectResult = []
    for (let eDTO of this) {
      objectResult.push(eDTO.toObject())
    }
    return objectResult
  }

  /**
    * KlineDTOArray toObject method
    *
    * @author jsb4ch <jsb4ch@jsmex.io>
    * @since 1.0.13
    */
  static fromObject (dataArray) {
    if (!(dataArray instanceof Array)) {
      throw new Error('Invalid dataArray')
    }
    for (let eDTO of dataArray) {
      if (!KlineDTO.isValid(eDTO)) {
        throw new Error('invalid element of data Array')
      }
    }
    let klineDTOArray = new KlineDTOArray(...dataArray)
    return klineDTOArray
  }
}

module.exports = KlineDTOArray
