const IDTOArray = global.IDTOArray || require('./IDTOArray')
const OrderDTO = global.OrderDTO || require('../OrderDTO')
const _ = global.lodash || require('../../../static_dependencies/lodash/lodash.js')

/**
  * OrderDTOArray class
  *
  * @author jsb4ch <jsb4ch@jsmex.io>
  * @since 1.0.9
  */
class OrderDTOArray extends IDTOArray {
  constructor (...args) {
    super(...args)
    this.validate()
  }

  /**
    * OrderDTOArray validate method
    *
    * @author jsb4ch <jsb4ch@jsmex.io>
    * @since 1.0.9
    */
  validate () {
    for (let order of this) {
      if (!OrderDTO.isValid(order)) {
        throw new Error('order is not order dto')
      }
    }
    return true
  }

  /**
  * OrderDTOArray save method
  *
  * @author jsb4ch <jsb4ch@jsmex.io>
  * @since 1.0.9
  */
  async save (db, filter = x => x) {
    if (!db) {
      throw new Error('invalid db')
    }
    if (typeof filter !== 'function') {
      throw new Error('invalid filter')
    }
    return db.getCollectionRepository('order').insertMany(this)
  }

  /**
  * OrderDTOArray load method
  *
  * @author jsb4ch <jsb4ch@jsmex.io>
  * @since 1.0.9
  */
  async load (db, query = {}) {
    if (!db) {
      throw new Error('invalid db')
    }
    let res = await db.getCollectionRepository('order').get(query) || []
    _.merge(this, res)
    return this
  }

  /**
  * OrderDTOArray remove method
  *
  * @author jsb4ch <jsb4ch@jsmex.io>
  * @since 1.0.9
  */
  async remove (db, filter = x => x) {
    if (!db) {
      throw new Error('invalid db')
    }
    if (typeof filter !== 'function') {
      throw new Error('invalid filter')
    }
    await db.getCollectionRepository('order').remove(this)
    return this.splice()
  }

  /**
    * OrderDTOArray toObject method
    *
    * @author jsb4ch <jsb4ch@jsmex.io>
    * @since 1.0.13
    */
  toObject () {
    let objectResult = []
    for (let eDTO of this) {
      objectResult.push(eDTO.toObject())
    }
    return objectResult
  }

  /**
    * OrderDTOArray toObject method
    *
    * @author jsb4ch <jsb4ch@jsmex.io>
    * @since 1.0.13
    */
  static fromObject (dataArray) {
    if (!(dataArray instanceof Array)) {
      throw new Error('Invalid dataArray')
    }
    for (let eDTO of dataArray) {
      if (!OrderDTO.isValid(eDTO)) {
        throw new Error('invalid element of data Array')
      }
    }
    let orderDTOArray = new OrderDTOArray(...dataArray)
    return orderDTOArray
  }
}

module.exports = OrderDTOArray
