const IDTOArray = global.IDTOArray || require('./IDTOArray')
const TransactionDTO = global.TransactionDTO || require('../TransactionDTO')
const _ = global.lodash || require('../../../static_dependencies/lodash/lodash.js')

/**
  * TransactionDTOArray class
  *
  * @author jsb4ch <jsb4ch@jsmex.io>
  * @since 1.0.9
  */
class TransactionDTOArray extends IDTOArray {
  constructor (...args) {
    super(...args)
    this.validate()
  }

  /**
    * TransactionDTOArray validate method
    *
    * @author jsb4ch <jsb4ch@jsmex.io>
    * @since 1.0.9
    */
  validate () {
    for (let transaction of this) {
      if (!TransactionDTO.isValid(transaction)) {
        throw new Error('transaction is not transaction dto')
      }
    }
    return true
  }

  /**
  * TransactionDTOArray save method
  *
  * @author jsb4ch <jsb4ch@jsmex.io>
  * @since 1.0.9
  */
  async save (db, filter = x => x) {
    if (!db) {
      throw new Error('invalid db')
    }
    if (typeof filter !== 'function') {
      throw new Error('invalid filter')
    }
    return db.getCollectionRepository('transaction').insertMany(this)
  }

  /**
  * TransactionDTOArray load method
  *
  * @author jsb4ch <jsb4ch@jsmex.io>
  * @since 1.0.9
  */
  async load (db, query = {}) {
    if (!db) {
      throw new Error('invalid db')
    }
    let res = await db.getCollectionRepository('transaction').get(query) || []
    _.merge(this, res)
    return this
  }

  /**
  * TransactionDTOArray remove method
  *
  * @author jsb4ch <jsb4ch@jsmex.io>
  * @since 1.0.9
  */
  async remove (db, filter = x => x) {
    if (!db) {
      throw new Error('invalid db')
    }
    if (typeof filter !== 'function') {
      throw new Error('invalid filter')
    }
    await db.getCollectionRepository('transaction').remove(this)
    return this.splice()
  }

  /**
    * TransactionDTOArray toObject method
    *
    * @author jsb4ch <jsb4ch@jsmex.io>
    * @since 1.0.13
    */
  toObject () {
    let objectResult = []
    for (let eDTO of this) {
      objectResult.push(eDTO.toObject())
    }
    return objectResult
  }

  /**
    * TransactionDTOArray toObject method
    *
    * @author jsb4ch <jsb4ch@jsmex.io>
    * @since 1.0.13
    */
  static fromObject (dataArray) {
    if (!(dataArray instanceof Array)) {
      throw new Error('Invalid dataArray')
    }
    for (let eDTO of dataArray) {
      if (!TransactionDTO.isValid(eDTO)) {
        throw new Error('invalid element of data Array')
      }
    }
    let transactionDTOArray = new TransactionDTOArray(...dataArray)
    return transactionDTOArray
  }
}

module.exports = TransactionDTOArray
