global.fetch = global.fetch || require('../../static_dependencies/node-fetch')
const ccxt = global.ccxtModule || require('../../static_dependencies/ccxt/ccxtbundle.js')
global.log = global.log || console.log

/**
  * IExchange class
  *
  * @author jsb4ch <jsb4ch@jsmex.io>
  * @since 1.0.0
  */
class IExchange {
  constructor (ccxtExchangeName, options = {}, override = {}) {
    // ccxt
    this.ccxt = ccxt
    const CcxtExchange = ccxt[ccxtExchangeName]
    if (typeof CcxtExchange === 'undefined') {
      const error = { code: 2009, isCritical: true }
      throw error
    }

    this.ccxtInstance = new CcxtExchange(options)
    if (options.apiDomain) {
      this.ccxtInstance.urls.api = `${options.protocol || 'https'}://${options.apiDomain}${options.port ? `:${options.port}` : ''}`
    }
    if (options.wwwDomain) {
      this.ccxtInstance.urls.www = options.wwwDomain
    }
    Object.assign(this.ccxtInstance, override)

    // errors
    if (this.balance === 'undefined') {
      const error = { code: 2010, isCritical: true }
      throw error
    }

    if (this.putOrder === 'undefined') {
      const error = { code: 2011, isCritical: true }
      throw error
    }

    if (this.cancelOrder === 'undefined') {
      const error = { code: 2012, isCritical: true }
      throw error
    }

    if (this.getTransactions === 'undefined') {
      const error = { code: 2013, isCritical: true }
      throw error
    }

    // warnings
    if (this.withdrawal === 'undefined') {
      global.log({ message: 'You should override withdrawal', type: 'WARN' })
    }

    if (this.depositAddress === 'undefined') {
      global.log({ message: 'You should override depositAddress', type: 'WARN' })
    }

    if (this.data === 'undefined') {
      global.log({ message: 'You should override data', type: 'WARN' })
    }
  }
}

module.exports = IExchange
